;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the               *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA     *
;*                                                                           *
;*****************************************************************************
; ________________________________________________
; |            RGB frame main program            |
; |______________________________________________|
; |                   Src File                   |
; |______________________________________________|
; |   Type      : ASM                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|


    LIST p=PIC16F15325, c=80, r=HEX, x=ON

#include <p16f15325.inc>

; CONFIG1
; External Oscillator mode selection bits (Oscillator not enabled)
; Power-up default value for COSC bits (HFINTOSC with OSCFRQ= 32 MHz and CDIV = 1:1)
; Clock Out Enable bit (CLKOUT function is disabled; i/o or oscillator function on OSC2)
; Clock Switch Enable bit (Writing to NOSC and NDIV is allowed)
; Fail-Safe Clock Monitor Enable bit (FSCM timer enabled)
; __config 0x3F8C
 __CONFIG _CONFIG1, _FEXTOSC_OFF & _RSTOSC_HFINT32 & _CLKOUTEN_OFF & _CSWEN_ON & _FCMEN_ON

; CONFIG2
; Master Clear Enable bit (MCLR pin is Master Clear function)
; Power-up Timer Enable bit (PWRT enabled)
; Low-Power BOR enable bit (ULPBOR disabled)
; Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
; Brown-out Reset Voltage Selection (Brown-out Reset Voltage (VBOR) set to 1.9V on LF, and 2.45V on F Devices)
; Zero-cross detect disable (Zero-cross detect circuit is disabled at POR.)
; Peripheral Pin Select one-way control (The PPSLOCK bit can be set and cleared repeatedly by software)
; Stack Overflow/Underflow Reset Enable bit (Stack Overflow or Underflow will cause a reset)
; __config 0x37FD
 __CONFIG _CONFIG2, _MCLRE_ON & _PWRTE_ON & _LPBOREN_OFF & _BOREN_ON & _BORV_LO & _ZCD_OFF & _PPS1WAY_OFF & _STVREN_ON

; CONFIG3
; WDT Period Select bits (Divider ratio 1:65536; software control of WDTPS)
; WDT operating mode (WDT disabled)
; WDT Window Select bits (window always open (100%); software control; keyed access not required)
; WDT input clock selector (Software Control)
; __config 0x3F9F
 __CONFIG _CONFIG3, _WDTCPS_WDTCPS_31 & _WDTE_OFF & _WDTCWS_WDTCWS_7 & _WDTCCS_SC

; CONFIG4
; Boot Block Size Selection bits (512 words boot block size)
; Boot Block Enable bit (Boot Block disabled)
; SAF Enable bit (SAF enabled)
; Application Block Write Protection bit (Application Block not write protected)
; Boot Block Write Protection bit (Boot Block not write protected)
; Configuration Register Write Protection bit (Configuration Register not write protected)
; Storage Area Flash Write Protection bit (SAF not write protected)
; Low Voltage Programming Enable bit (Low Voltage programming enabled. MCLR/Vpp pin function is MCLR.)
; __config 0x3FEF
 __CONFIG _CONFIG4, _BBSIZE_BB512 & _BBEN_OFF & _SAFEN_ON & _WRTAPP_OFF & _WRTB_OFF & _WRTC_OFF & _WRTSAF_OFF & _LVP_ON

; CONFIG5
; UserNVM Program memory code protection bit (UserNVM code protection disabled)
; __config 0x3FFF
 __CONFIG _CONFIG5, _CP_OFF

#include "macro_16F.inc"

;***************************************************************************
;                               VARIABLES
;***************************************************************************
    UDATA
;---------------------------------------------------------------------Bank 0
        BANK_0
	#include "src/def_system.inc"
	#include "src/def_saf.inc"
	#include "src/def_timer.inc"
	#include "src/def_io.inc"
	#include "src/def_pwm.inc"
	#include "src/def_dmx.inc"
	BYTE LAST			;last byte bank 0

 ;***************************************************************************
;                                   RESET
;***************************************************************************

RES_VECT  CODE 0x0000			;RESET VECTOR
	GOTO SYS_STARTUP


;***************************************************************************
;                                INTERRUPTS
;***************************************************************************

INT_VECT CODE 0x0004			;INTERRUPT_VECTOR
	CLI				;Disable all IRQs
	BCF PCLATH,3			;Page 0
Tmr1_Int				;Main timer management
	BANKSEL PIR4			;Bank 14
	BTFSS PIR4,TMR1IF		;Check if TIMER1 is in OVF
	GOTO Tmr0_Int			; NO
	BCF PIR4,TMR1IF			; YES, clear Timer1 IRQ flag
	BANKSEL T1CON			;Bank 4
	BCF T1CON,TMR1ON		;Disable Timer1
	ADDLF c_TMR1_TIME_H,TMR1H	;Reset Timer1 duration
	ADDLF c_TMR1_TIME_L,TMR1L	;
	BSF T1CON,TMR1ON		;Enable Timer1
	BANKSEL b_TMR_STAT		;Bank 0
	BSF f_IRQ_TIMER1_ON		;Set Timer1 IRQ flag
Tmr0_Int				;DMX timer management
	BANKSEL PIR0			;Bank 14
	BTFSS PIR0,TMR0IF		;Check if TIMER0 is in OVF
	GOTO Ser_Int			; NO
	BCF PIR0,TMR0IF			; YES, clear Timer0 IRQ flag
	BCF PIE0,TMR0IE			;Disable Timer0 IRQ
	BANKSEL T0CON0			;Bank 11
	BCF T0CON0,T0EN			;Stop Timer0
	BANKSEL b_TMR_STAT		;Bank 0
	BSF f_IRQ_TIMER0_ON		;Set Timer0 IRQ flag
Ser_Int
	BANKSEL PIR3			;Bank 14
	BTFSS PIR3,RC1IF		;Check if serial port received a byte
	GOTO End_Int			; NO
	BCF PIR3,RC1IF			; YES, clear RX IRQ flag
	BANKSEL b_DMX_CTRL		; Bank 14
	BTFSS f_DMX_INH			; Check if DMX inhibit is enabled
	GOTO End_Int			;  NO, leave
	BSF PCLATH,3			;  YES, Page 1
	CALL Dmx_Slave			;  Manage the reception
	BCF PCLATH,3			;  Page 0
End_Int	
	BANKSEL PORTA			;Bank 0
	RETFIE

MAIN_PROG CODE
;***************************************************************************
;                             SYSTEM START-UP
;***************************************************************************
SYS_STARTUP
	CLI				;Disable all IRQs
	CALL Sys_Init			;System init
	CALL Timer_Init			;Timer init
	CALL Pwm_Init			;PWM init
	BSF PCLATH,3			;Page 1
	CALL Io_Init			;I/O init
	CALL Dmx_Init			;DMX init
	CALL SAF_Read			;Recall parameters from SAF (if saved)
	CALL Dmx_Addr_BCD		;Convert DMX address into BCD for menu/display use
	CALL Dmx_StartR			;Start DMX receive if needed
	BCF PCLATH,3			;Page 0
;add other init functions here

;***************************************************************************
;                           MAIN PROGRAM
;***************************************************************************
PROGRAM
	BANKSEL T1CON			;Bank 4
	BSF T1CON,TMR1ON		;Enable Timer1
	BSF INTCON,PEIE			;Enable peripheral interrupts
	SEI				;Enable all interrupts
	BANKSEL b_TMR_STAT		;Bank 0

;MAIN LOOP. MCU keeps cycling here
Main_Loop
	BANKSEL b_TMR_STAT		;Bank 0
	BTFSC f_IRQ_TIMER1_ON		;Check if Timer1 IRQ
	GOTO Fast_Ck			; Yes
	BTFSC f_IRQ_TIMER0_ON		; No, check if Timer0 IRQ
	GOTO Main_DX			;  Yes
	GOTO Main_Loop			;  No, repeat

;*******************************************************
;FAST LOOP - 10ms
;*******************************************************
Fast_Ck	
	BCF f_IRQ_TIMER1_ON		;Clear Timer1 IRQ flag
;add subs here
	BSF PCLATH,3			;Page 1
	CALL Push_Read			;Pushbutton control
	CALL Dmx_Rxck			;DMX receive check (reset RX errors)
	BCF PCLATH,3			;Page 0
	CALL RGB_OnOff			;Soft-On/Off RGB management

;*******************************************************
;SLOW LOOP 1 - 40 ms
;*******************************************************
Low_Ck1
	DECFSZ b_TMR3_CNT,f		;Decrement slow timer 3
	GOTO Main_DX			; if not zero, skip to DMX

	MOVLF c_TMR3_DIV,b_TMR3_CNT	;zero, reloads timer
;add subs here
	CALL RGB_Manager		;RGB management
	CALL RGB_Cycle			;RGB cycle control
	INCF b_DMX_ERR,f		;Increment DMX receive error counter (control is not done here)

;*******************************************************
;SLOW LOOP 2 - 120 ms
;*******************************************************
Low_Ck2
	DECFSZ b_TMR4_CNT,f		;Decrement slow timer 4
	GOTO Main_DX			; if not zero, skip to DMX

	MOVLF c_TMR4_DIV,b_TMR4_CNT	;zero, reloads timer
;add subs here
	CALL Sys_Menu			;System menu
	BSF PCLATH,3			;Page 1
	CALL SAF_Write			;SAF save data
	BCF PCLATH,3			;Page 0
	INCF b_SW_SLOW,f		;Increment slow push counter (control is not done here)

;*******************************************************
;DMX master loop
;*******************************************************
Main_DX	
	BSF PCLATH,3			;Page 1
	CALL Dmx_Master			;DMX master sub
	BCF PCLATH,3			;Page 0
	Goto Main_Loop			;Go back to loop start


;***************************************************************************
;                               INCLUDE FILES
;***************************************************************************
;---------------------------------------------------------------------Page 0
#include "src/asm_system.asm"
#include "src/asm_timer.asm"
#include "src/asm_pwm.asm"
;---------------------------------------------------------------------Page 1
    CODE 0x0800
#include "src/asm_io.asm"
#include "src/asm_saf.asm"
#include "src/asm_dmx.asm"

        END
