;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |      Include for RGB frame PWM control       |
; |______________________________________________|
; |   Include File                               |
; |______________________________________________|
; |   Type      : INC                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________Constants
c_PWM_PER	EQU	0xF9		;PWM period (with 32MHz and prescaler 16, >> 2kHz)
c_PWM3_CON	EQU	B'10000000'	;PWM3 enabled, output level low, active high
c_PWM4_CON	EQU	B'10000000'	;PWM4 enabled, output level low, active high   
c_PWM5_CON	EQU	B'10000000'	;PWM5 enabled, output level low, active high

c_RGB_FASTER	EQU	0x01		;Faster RGB cycle (60 sec)
c_RGB_FAST	EQU	0x03		;Fast RGB cycle (3 min)
c_RGB_MEDIUM	EQU	0x07		;Medium RGB cycle (7 min)
c_RGB_SLOW	EQU	0x0F		;Slow RGB cycle (15 min)
c_RGB_SLOWER	EQU	0x1E		;Slower RGB cycle (30 min)

c_SPD_MASK	EQU	B'11100000'	;Mask to clear all RGB speeds

c_TMR_CORR	EQU	0x03		;Init value for color correction timer
c_TMR_COR2	EQU	0x02		;Reload value for color correction timer	
c_2C_CORR	EQU	0x7F		;Color correction for 2 colors
c_3C_CORR	EQU	0xA9		;Color correction for 3 colors

;Fixed colors
;RED
c_RGB_REDR	EQU	0xFF
c_RGB_REDG	EQU	0x00
c_RGB_REDB	EQU	0x00
;RED - LOW INTENSITY
c_RGB_LREDR	EQU	0x10
c_RGB_LREDG	EQU	0x00
c_RGB_LREDB	EQU	0x00
;RED, GREEN 25%
c_RGB_RG25R	EQU	0xCC
c_RGB_RG25G	EQU	0x32
c_RGB_RG25B	EQU	0x00
;RED, GREEN 50%
c_RGB_RG50R	EQU	0xAA
c_RGB_RG50G	EQU	0x54
c_RGB_RG50B	EQU	0x00
;RED, GREEN 75%
c_RGB_RG75R	EQU	0x91
c_RGB_RG75G	EQU	0x6D
c_RGB_RG75B	EQU	0x00
;YELLOW
c_RGB_YLWR	EQU	0x7F
c_RGB_YLWG	EQU	0x7F
c_RGB_YLWB	EQU	0x00
;YELLOW - LOW INTENSITY
c_RGB_LYLWR	EQU	0x08
c_RGB_LYLWG	EQU	0x08
c_RGB_LYLWB	EQU	0x00
;GREEN, RED 75%
c_RGB_GR75R	EQU	0x6D
c_RGB_GR75G	EQU	0x91
c_RGB_GR75B	EQU	0x00
;GREEN, RED 50%
c_RGB_GR50R	EQU	0x54
c_RGB_GR50G	EQU	0xAA
c_RGB_GR50B	EQU	0x00
;GREEN, RED 25%
c_RGB_GR25R	EQU	0x32
c_RGB_GR25G	EQU	0xCC
c_RGB_GR25B	EQU	0x00
;GREEN
c_RGB_GRNR	EQU	0x00
c_RGB_GRNG	EQU	0xFF
c_RGB_GRNB	EQU	0x00
;GREEN - LOW INTENSITY
c_RGB_LGRNR	EQU	0x00
c_RGB_LGRNG	EQU	0x10
c_RGB_LGRNB	EQU	0x00
;GREEN, BLUE 25%
c_RGB_GB25R	EQU	0x00
c_RGB_GB25G	EQU	0xCC
c_RGB_GB25B	EQU	0x32
;GREEN, BLUE 50%
c_RGB_GB50R	EQU	0x00
c_RGB_GB50G	EQU	0xAA
c_RGB_GB50B	EQU	0x54
;GREEN, BLUE 75%
c_RGB_GB75R	EQU	0x00
c_RGB_GB75G	EQU	0x91
c_RGB_GB75B	EQU	0x6D
;AZURE
c_RGB_AZRR	EQU	0x00
c_RGB_AZRG	EQU	0x7F
c_RGB_AZRB	EQU	0x7F
;AZURE - LOW INTENSITY
c_RGB_LAZRR	EQU	0x00
c_RGB_LAZRG	EQU	0x08
c_RGB_LAZRB	EQU	0x08
;BLUE, GREEN 75%
c_RGB_BG75R	EQU	0x00
c_RGB_BG75G	EQU	0x6D
c_RGB_BG75B	EQU	0x91
;BLUE, GREEN 50%
c_RGB_BG50R	EQU	0x00
c_RGB_BG50G	EQU	0x54
c_RGB_BG50B	EQU	0xAA
;BLUE, GREEN 25%
c_RGB_BG25R	EQU	0x00
c_RGB_BG25G	EQU	0x32
c_RGB_BG25B	EQU	0xCC
;BLUE
c_RGB_BLUR	EQU	0x00
c_RGB_BLUG	EQU	0x00
c_RGB_BLUB	EQU	0xFF
;BLUE - LOW INTENSITY
c_RGB_LBLUR	EQU	0x00
c_RGB_LBLUG	EQU	0x00
c_RGB_LBLUB	EQU	0x10
;BLUE, RED 25%
c_RGB_BR25R	EQU	0x32
c_RGB_BR25G	EQU	0x00
c_RGB_BR25B	EQU	0xCC
;BLUE, RED 50%
c_RGB_BR50R	EQU	0x54
c_RGB_BR50G	EQU	0x00
c_RGB_BR50B	EQU	0xAA
;BLUE, RED 75%
c_RGB_BR75R	EQU	0x6D
c_RGB_BR75G	EQU	0x00
c_RGB_BR75B	EQU	0x91
;VIOLET
c_RGB_VLTR	EQU	0x7F
c_RGB_VLTG	EQU	0x00
c_RGB_VLTB	EQU	0x7F
;VIOLET - LOW INTENSITY
c_RGB_LVLTR	EQU	0x08
c_RGB_LVLTG	EQU	0x00
c_RGB_LVLTB	EQU	0x08
;RED, BLUE 75%
c_RGB_RB75R	EQU	0x91
c_RGB_RB75G	EQU	0x00
c_RGB_RB75B	EQU	0x6D
;RED, BLUE 50%
c_RGB_RB50R	EQU	0xAA
c_RGB_RB50G	EQU	0x00
c_RGB_RB50B	EQU	0x54
;RED, BLUE 25%
c_RGB_RB25R	EQU	0xCC
c_RGB_RB25G	EQU	0x00
c_RGB_RB25B	EQU	0x32
;WHITE
c_RGB_WHTR	EQU	0x65		;Correction for a warmer white (0x55)
c_RGB_WHTG	EQU	0x55
c_RGB_WHTB	EQU	0x45		;Correction for a warmer white (0x55)
;WHITE - LOW INTENSITY
c_RGB_LWHTR	EQU	0x06		;Correction for a warmer white (0x05)
c_RGB_LWHTG	EQU	0x05
c_RGB_LWHTB	EQU	0x04		;Correction for a warmer white (0x05)


;_______________________________________________________Variables
	BYTE b_PWM_RED			;PWM will be managed with 8 bit resolution (LSBs at 0)
	BYTE b_PWM_RBK			;
	BYTE b_PWM_GRN			;
	BYTE b_PWM_GBK			;
	BYTE b_PWM_BLU			;
	BYTE b_PWM_BBK			;
	BYTE b_RGB_SPD			;RGB cycle speed
	BYTE b_RGB_CNT			;RGB cycle counter 1
	BYTE b_RGB_CNT2			;RGB cycle counter 2
	BYTE b_RGB_BLINK		;RGB blink timer
	BYTE b_RGB_STEP			;RGB cycle step
	BYTE b_RGB_STEPB		;RGB cycle step backup
	BYTE b_RGB_CORR			;RGB duty correction for keeping same power with 1, 2 or 3 colors
	WORD w_RGB_CTRL			;RGB control flags
	BYTE b_RGB_MODE			;RGB cycle mode
	BYTE b_RGB_COLOR		;RGB fixed color index

;RGB_MODE
;MODE  RED GREEN BLUE
; 0	ERROR
; 1	+    0    0
; 2	+    1    0
; 3	+    0    1
; 4	+    +    0
; 5	+    +    1
; 6	+    0    +
; 7	+    1    +
; 8	-    0    0
; 9	-    1    0
;10	-    0    1
;11	-    -    0
;12	-    -    1
;13	-    0    -
;14	-    1    -
;15	0    +    0
;16	1    +    0
;17     0    +    1
;18     0    +    +
;19     1    +    +
;20     0    -    0
;21     1    -    0
;22     0    -    1
;23     0    -    -
;24     1    -    -
;25     0    0    +
;26     1    0    +
;27     0    1    +
;28     0    0    +
;29     1    0    +
;30     0    1    +
;31     F    F    F
;32-255	ERROR

;_______________________________________________________Flags
#define f_RGB_SPD1	w_RGB_CTRL,0	;RGB speed Faster
#define f_RGB_SPD2	w_RGB_CTRL,1	;RGB speed Fast
#define f_RGB_SPD3	w_RGB_CTRL,2	;RGB speed Medium
#define f_RGB_SPD4	w_RGB_CTRL,3	;RGB speed Slow
#define f_RGB_SPD5	w_RGB_CTRL,4	;RGB speed Slower
#define f_RGB_BLNK	w_RGB_CTRL,5	;RGB blink mode
#define f_RGB_DONE	w_RGB_CTRL,6	;RGB cycle done
#define f_RGB_ERR	w_RGB_CTRL,7	;RGB cycle error

#define f_RGB_RUN	w_RGB_CTRL+1,0	;RGB cycle activation
#define f_RGB_END	w_RGB_CTRL+1,1	;RGB soft start status
#define f_RGB_ON	w_RGB_CTRL+1,2	;RGB soft start to ON
#define f_RGB_OFF	w_RGB_CTRL+1,3	;RGB soft start to OFF
