;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |     Include for RGB frame timer control      |
; |______________________________________________|
; |   Include File                               |
; |______________________________________________|
; |   Type      : INC                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________Constants
c_TMR0_CON0	EQU	B'00010000'	;Timer0 disabled, 16bit, postcaler 1:1
c_TMR0_CON1	EQU	B'01000000'	;Timer0 on internal CLK with Fosc/4, sync, prescaler 1:1
	
c_TMR1_CON	EQU	B'00010000'	;Timer1 on internal CLK with 1:2 prescaler
c_TMR1_GCON	EQU	B'00000000'	;Timer1 gate control disable (always counting)
c_TMR1_CLK	EQU	B'00000001'	;Timer1 clock to Fosc/4
c_TMR1_GATE	EQU	B'00000000'	;Timer1 gate select (unused, gate disabled)

c_TMR2_CON	EQU	B'01000000'	;Timer2 disabled, on internal CLK with
					; prescaler 1:16 and postscaler 1:1
c_TMR2_CLK	EQU	B'00000001'	;Timer2 clock to Fosc/4
c_TMR2_HLT	EQU	B'00000000'	;Timer2 prescaler not sync, rising edge of clock,
					; ON not sync, free running with sw gate
c_TMR2_RST	EQU	B'00000000'	;Timer2 reset on T2INPPS

;_______________________Main Timer
;System main timer (0x9C40 = 10 ms with CLK @ 32MHz)
; Since the timer increments up to FFFF and then goes into IRQ, I must subtract
; the needed value from FFFF and use the result
c_TMR1_TIME_H   EQU	0x63            ;Main timer value
c_TMR1_TIME_L   EQU	0xBF            ;

;_______________________DMX related timers
;Dmx_Master uses 2 instr between clearing pin and setting timer
; + IRQ requires 30 + 9-15 to reach Dmx_Master sub
; + 7 to jump to MAB routines and set the pin --> 48/54 = 6/6.75us (I'll use 7)
c_TMR0_BRKL	EQU	0xB7		;169us (Break = 176us-7us)
c_TMR0_BRKH	EQU	0xFA		;

;MAB is minimum 12us and maximum 1s. This will make it around 20us
c_TMR0_MABL	EQU	0x9F		;12us (Mark-After-Break)
c_TMR0_MABH	EQU	0xFF		;

;_______________________Dividers
c_TMR3_DIV      EQU	0x04		;slow cycle 1 delay (40ms)
c_TMR4_DIV      EQU	0x03		;slow cycle 2 delay (120ms)
c_TMR_BLK1	EQU	0x02		;blink timer 80ms
c_TMR_BLK2	EQU	0x05		;blink timer 200ms
c_TMR_BLK3	EQU	0x0A		;blink timer 320ms
c_TMR_BLK8	EQU	0x14		;blink timer 800ms

;_______________________________________________________Variabili
	BYTE b_TMR3_CNT			;Slow cycle 1 counter
	BYTE b_TMR4_CNT			;Slow cycle 2 counter
	BYTE b_TMR_STAT			;Timers status

;_______________________________________________________Flags
#define f_IRQ_TIMER1_ON b_TMR_STAT,0	;Timer1 IRQ flag
#define f_IRQ_TIMER0_ON b_TMR_STAT,1	;Timer0 IRQ flag
